#!/bin/perl
use strict;
use warnings;

#
# wrapping script for dmd/ldc2 + flymake on CygwinEmacs   by TANE
#
# 使い方
#   (0)dmdもしくはldc2コンパイラを使用できるようにしておく。
#
#   (1)このスクリプトを適当な場所(例えば/usr/local/binなど)に置く。
#      このとき、ファイル名をdmdfm.plで置くと コンパイラにdmdを使用し、
#      ファイル名を ldc2fm.pl で置くと コンパイラにldc2を使用するように
#      なります。片方をシンボリックリンクで置いても構いません。
#
#   (2) .emacs にflymakeとd-modeの設定を追加する。
#;;-----------------------------
#(require 'd-mode)
#(require 'flymake)
#
#(defun flymake-D-init ()
#  (let* ((temp-file (flymake-init-create-temp-buffer-copy
#                     'flymake-create-temp-inplace))
#         (local-file (file-relative-name
#                      temp-file
#                      (file-name-directory buffer-file-name))))
#    (list "make"
#          (list "-s"
#                "-C"
#                (file-name-directory buffer-file-name)
#                (concat "CHK_SOURCES=" local-file)
#                "SYNTAX_CHECK_MODE=1"
#                "check-syntax"))
#    ))
#(add-to-list 'flymake-allowed-file-name-masks
#             '(".+\\.d$" flymake-D-init
#               flymake-simple-cleanup flymake-get-real-file-name))
#(add-to-list 'flymake-proc-err-line-patterns
#             '("^\\([^ :]+\\)(\\([0-9]+\\)): \\(.*\\)$" 1 2 nil 3))
#
#(add-hook 'd-mode-hook 'set-flymake-mode)
#;;-----------------------------
#
#   (3)ソースコードと同じディレクトリにMakefileを用意し、check-syntax
#      ルールを追加する。以下の例では ldc2を使用します。
###-----------------------------
#LANG=C
#DC      = ldc2 #or dmd
#DCFM    = /usr/local/bin/ldc2fm.pl #or dmdfm.pl
#DFLAGS  = -O -I.
#
#%:%.d
#	$(DC) $(DFLAGS) $<
#
#.PHONY: check-syntax
#
#check-syntax:
#ifeq ($(filter -x,$(CHK_SOURCES)),-x)
#  ifeq ($(filter -x c,$(CHK_SOURCES)),-x c)
#	$(CC) $(CFLAGS) -fsyntax-only $(CHK_SOURCES)
#  endif
#  ifeq ($(filter -x d,$(CHK_SOURCES)),-x d)
#	$(DCFM) $(DFLAGS) $(CHK_SOURCES) -o-
#  endif
#else
#  ifeq ($(suffix $(CHK_SOURCES)),.c)
#	$(CC) $(CFLAGS) -fsyntax-only $(CHK_SOURCES)
#  endif
#  ifeq ($(suffix $(CHK_SOURCES)),.d)
#	$(DCFM) $(DFLAGS) $(CHK_SOURCES) -o-
#  endif
#endif
###-----------------------------
#   (4)Makefileと同じディレクトリのD言語ソースコードを編集すると
#      ldc2でエラーチェックが行なわれるようになります。
#

my $version="dmdfm.pl 2024/3/24a";

my $DC="dmd";

if( $0=~/ldc2/ ){
    $DC="ldc2";
}

my @args;
foreach my $e (@ARGV){
    if( $e=~/\.d$/ ){
        my $fname=`cygpath -w $e`;
        chomp($fname);
        $fname =~ s/\\/\\\\/g;
        push(@args,$fname);
    }else{
        push(@args,$e);
    }
}

my $cmdline="$DC @args 2>&1";
my $result = `$cmdline`;
my $exitcode = $?;

my @lines = split(/\r\n/, $result);

foreach my $l (@lines){
    if( $l=~/([^ ]+)(\([0-9]+\): .*$)/ ){
        my $orgfname=$1;
        my $orgmsg  =$2;
        $orgfname =~ s/\\/\\\\/g;
        my $fname=`cygpath $orgfname`;
        chomp($fname);
        print $fname, $orgmsg, "\n";
    }else{
        print $l,"\n";
    }
}

exit($exitcode >> 8);
